/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * --/COPYRIGHT--
 */

#include <ti/devices/msp432p4xx/inc/msp.h>
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>
#include <stdio.h>

/* Standard Includes */
#include <stdint.h>
#include <string.h>


/* Clock system frequencies */
/* Clock system frequencies */
#define MCLK_FREQUENCY      32000000
#define SMCLK_FREQUENCY     16000000
#define SPI_FREQUENCY       16000000

#define NUMBER_OF_SAMPLES   	256

/* SPI Master Configuration Parameter */

const eUSCI_SPI_MasterConfig spiMasterConfig =
{
    EUSCI_B_SPI_CLOCKSOURCE_SMCLK,              // SMCLK Clock Source
	SMCLK_FREQUENCY,                            // SMCLK = DCO
	SPI_FREQUENCY,                              // SPICLK = DCO/divider
	EUSCI_B_SPI_LSB_FIRST,                      // MSB First
	EUSCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,    // Phase
	EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,  // High polarity
	EUSCI_B_SPI_3PIN                            // 3Wire SPI Mode
};

uint8_t dmaTable[NUMBER_OF_SAMPLES];

/* DMA Control Table */
#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN(MSP_EXP432P401RLP_DMAControlTable, 1024)
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment=1024
#elif defined(__GNUC__)
__attribute__ ((aligned (1024)))
#elif defined(__CC_ARM)
__align(1024)
#endif
static DMA_ControlTable MSP_EXP432P401RLP_DMAControlTable[16];  // 8 primary and 8 alternate

int main(void)
{
    uint16_t ii;
    /* Halting WDT  */
    MAP_WDT_A_holdTimer();
    MAP_Interrupt_disableMaster();
    //	MAP_Interrupt_enableSleepOnIsrExit();
    // SCB->SCR |= SCB_SCR_SLEEPDEEP_Pos;

    for(ii=0;ii<NUMBER_OF_SAMPLES;ii++)
    {
        dmaTable[ii] = ii;
    }

    /*
     * At VCore1 max MCLK is 48Mhz and Max Periph Clock is 24Mhz
     */
    MAP_PCM_setPowerState(PCM_AM_LDO_VCORE1);

    MAP_FlashCtl_setWaitState(FLASH_BANK0, 1);
    MAP_FlashCtl_setWaitState(FLASH_BANK1, 1);

    MAP_FPU_enableModule();

    MAP_CS_setDCOFrequency(MCLK_FREQUENCY);

    /*
     * Setting up clocks
     * MCLK = 48Mhz
     * SMCLK = 48Mhz
     */
    MAP_CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    MAP_CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_2);

    /* Configuring DMA module */
    MAP_DMA_enableModule();
    MAP_DMA_setControlBase(MSP_EXP432P401RLP_DMAControlTable);

	 /* Disabling channel attributes */
	MAP_DMA_disableChannelAttribute(DMA_CH0_EUSCIB0TX0,
								 UDMA_ATTR_USEBURST |
								 UDMA_ATTR_HIGH_PRIORITY |
								 UDMA_ATTR_REQMASK);

    /* Setting Control Indexes. In this case we will set the source of the
     * DMA transfer to TI_onebDACTable
     *  and the destination to the
     * EUSCI transmit buffer.
     */
    MAP_DMA_setChannelControl(UDMA_PRI_SELECT | DMA_CH0_EUSCIB0TX0,
        UDMA_SIZE_8 | UDMA_SRC_INC_8 | UDMA_DST_INC_NONE | UDMA_ARB_1);
    MAP_DMA_setChannelTransfer(UDMA_PRI_SELECT | DMA_CH0_EUSCIB0TX0,
        UDMA_MODE_BASIC, dmaTable,
		(void*) MAP_SPI_getTransmitBufferAddressForDMA(EUSCI_B0_BASE),
		NUMBER_OF_SAMPLES);

	/* Selecting P1.5 and P1.6 in SPI mode */
	MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1,
			GPIO_PIN5 | GPIO_PIN6 , GPIO_PRIMARY_MODULE_FUNCTION);

	/* Configuring SPI in 3wire master mode */
	MAP_SPI_initMaster(EUSCI_B0_BASE, &spiMasterConfig);

    /*
     * Enable SPI module
     */
    MAP_SPI_enableModule(EUSCI_B0_BASE);

    /* Configuring P1.0 as output for debug */
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0);

    /*
     * Configure P2.0 as the chip select - active low
     * P2.1, and P2.2 for debug
     */
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN1|GPIO_PIN2);
    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN0);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2);

    /* Assigning/Enabling Interrupts */
//    MAP_DMA_assignInterrupt(DMA_INT1, 0);
//    MAP_Interrupt_enableInterrupt(INT_DMA_INT1);
	MAP_DMA_assignChannel(DMA_CH0_EUSCIB0TX0);

//    MAP_Interrupt_enableMaster();
	/*
	 * set CS low and start transmission via spi
	 */
	MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0);
    MAP_DMA_enableChannel(0);

	while(1)
	{
	    __no_operation();
        P1->OUT ^= BIT0;
	}
}

/* Completion interrupt for DMA */
//__attribute__((ramfunc))
//void DMA_INT1_IRQHandler(void)
//{
//
//	P2->OUT |= BIT0;
//
//
//    	/*
//    	 * After completion need to reset primary to ping-pong mode.
//    	 * Primary Channel0 control is at offset 0x008, bits 0-2.
//    	 * Set the size to the number of BYTE_NUMBER_OF_DAC_SAMPLES
//    	 */
//    	MSP_EXP432P401RLP_DMAControlTable[0].control =
//    			(MSP_EXP432P401RLP_DMAControlTable[0].control & 0xff000000 ) |
//				(((BYTE_NUMBER_OF_DAC_SAMPLES)-1)<<4) | 0x03;
//
//    P2->OUT &= ~BIT0;
//}

